package controllers

import (
	"encoding/json"
	"eta_gn/eta_report/models"
	"eta_gn/eta_report/utils"
	"fmt"
	"github.com/beego/beego/v2/server/web"
	"net/http"
	"net/url"
)

type BaseOpenController struct {
	web.Controller
}

func (this *BaseOpenController) Prepare() {
	fmt.Println("enter prepare")
	method := this.Ctx.Input.Method()
	uri := this.Ctx.Input.URI()
	fmt.Println("Url:", uri)
	if method != "HEAD" {
		//校验签名
		nonce := this.Ctx.Input.Header("nonce")
		timestamp := this.Ctx.Input.Header("timestamp")
		appid := this.Ctx.Input.Header("appid")
		signature := this.Ctx.Input.Header("signature")

		if nonce == "" {
			errMsg := "随机字符串不能为空"
			this.JSON(models.BaseResponse{Ret: 400, Msg: errMsg, ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}

		if timestamp == "" {
			errMsg := "时间戳不能为空"
			this.JSON(models.BaseResponse{Ret: 400, Msg: errMsg, ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}

		if appid != utils.AppId {
			errMsg := "AppId错误，请核查"
			this.JSON(models.BaseResponse{Ret: 400, Msg: errMsg, ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}

		checkSign := utils.GetSign(nonce, timestamp)
		if signature != checkSign {
			utils.FileLog.Info(fmt.Sprintf("用户提交签名：%s；\n系统生成签名：%s\n", signature, checkSign))
			errMsg := "签名错误"
			this.JSON(models.BaseResponse{Ret: 401, Msg: errMsg, ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}
		if method != "GET" && method != "POST" {
			errMsg := "无效的请求方式"
			this.JSON(models.BaseResponse{Ret: 501, Msg: errMsg, ErrMsg: errMsg}, false, false)
			this.StopRun()
			return
		}
	} else {
		this.JSON(models.BaseResponse{Ret: 500, Msg: "系统异常，请联系客服!", ErrMsg: "method:" + method}, false, false)
		this.StopRun()
		return
	}
}

func (c *BaseOpenController) ServeJSON(encoding ...bool) {
	//所有请求都做这么个处理吧，目前这边都是做编辑、刷新逻辑处理（新增的话，并没有指标id，不会有影响）
	var (
		hasIndent   = false
		hasEncoding = false
	)
	if web.BConfig.RunMode == web.PROD {
		hasIndent = false
	}
	if len(encoding) > 0 && encoding[0] == true {
		hasEncoding = true
	}
	if c.Data["json"] == nil {
		//go utils.SendEmail("异常提醒:", "接口:"+"URI:"+c.Ctx.Input.URI()+";无返回值", utils.EmailSendToUsers)
		//body := "接口:" + "URI:" + c.Ctx.Input.URI() + ";无返回值"
		//go alarm_msg.SendAlarmMsg(body, 1)
		return
	}

	baseRes := c.Data["json"].(*models.BaseResponse)
	if baseRes != nil && baseRes.Ret != 408 {
		//body, _ := json.Marshal(baseRes)
		//var requestBody string
		//method := c.Ctx.Input.Method()
		//if method == "GET" {
		//	requestBody = c.Ctx.Request.RequestURI
		//} else {
		//	requestBody, _ = url.QueryUnescape(string(c.Ctx.Input.RequestBody))
		//}
		//if baseRes.Ret != 200 && baseRes.IsSendEmail {
		//	go utils.SendEmail(utils.APP_NAME_CN+"【"+utils.RunMode+"】"+"失败提醒", "URI:"+c.Ctx.Input.URI()+"<br/> "+"Params"+requestBody+" <br/>"+"ErrMsg:"+baseRes.ErrMsg+";<br/>Msg:"+baseRes.Msg+";<br/> Body:"+string(body)+"<br/>", utils.EmailSendToUsers)
		//}

		// 记录错误日志, 并清掉错误信息避免暴露给外部
		if baseRes.ErrMsg != "" {
			utils.FileLog.Info(baseRes.ErrMsg)
			baseRes.ErrMsg = ""
		}
	}
	c.JSON(c.Data["json"], hasIndent, hasEncoding)
}

func (c *BaseOpenController) JSON(data interface{}, hasIndent bool, coding bool) error {
	c.Ctx.Output.Header("Content-Type", "application/json; charset=utf-8")
	var content []byte
	var err error
	if hasIndent {
		content, err = json.MarshalIndent(data, "", "  ")
	} else {
		content, err = json.Marshal(data)
	}
	if err != nil {
		http.Error(c.Ctx.Output.Context.ResponseWriter, err.Error(), http.StatusInternalServerError)
		return err
	}
	ip := c.Ctx.Input.IP()
	requestBody, err := url.QueryUnescape(string(c.Ctx.Input.RequestBody))
	if err != nil {
		requestBody = string(c.Ctx.Input.RequestBody)
	}
	if requestBody == "" {
		requestBody = c.Ctx.Input.URI()
	}
	c.logUri(content, requestBody, ip)
	if coding {
		content = []byte(utils.StringsToJSON(string(content)))
	}
	return c.Ctx.Output.Body(content)
}

func (c *BaseOpenController) logUri(respContent []byte, requestBody, ip string) {
	var reqData interface{}
	err := json.Unmarshal([]byte(requestBody), &reqData)
	if err != nil {
		utils.ApiLog.Info("uri:%s, requestBody:%s, ip:%s", c.Ctx.Input.URI(), requestBody, ip)
	} else {
		utils.ApiLog.Info("uri:%s, requestBody:%s, ip:%s", c.Ctx.Input.URI(), requestBody, ip)
	}
	return
}
